package com.sundata.internalevaluation.calc.calcUnit;


import cn.hutool.extra.spring.SpringUtil;
import com.sundata.internalevaluation.calc.model.CalcResult;
import com.sundata.internalevaluation.calc.model.CalcUnit;
import com.sundata.internalevaluation.calc.model.finals.CalcType;
import com.sundata.internalevaluation.configuration.model.IndexModel;
import com.sundata.internalevaluation.configuration.model.IndexSetModel;
import com.sundata.internalevaluation.configuration.service.IndexConfigService;
import com.sundata.internalevaluation.configuration.service.IndexSetService;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

public class IndexListConfigCalcUnit extends CalcUnit {

    IndexSetModel indexLisModel;

    /**
     * 创建数据单元的绝对对象，对象必须包含如下参数
     *
     * @param calcCode    计算对象编号
     * @param calcName    计算对象名称
     * @param calcType    计算类型
     * @param initContext 计算单元初始化参数
     */
    public IndexListConfigCalcUnit(String calcCode, String calcName, CalcType calcType, Map<String, Object> initContext, IndexSetModel indexLisModel) {
        super(calcCode, calcName, calcType, initContext);
        this.indexLisModel = indexLisModel;
    }

    @Override
    public boolean isCalcFinished(String calculateInstanceNumber) {
        return false;
    }

    @Override
    public void initResultContext(String calculateInstanceNumber) {

    }

    @Override
    public List<CalcUnit> getSourceCalcUnits() {
        IndexSetService indexSetService = SpringUtil.getBean(IndexSetService.class);
        IndexConfigService indexConfigService = SpringUtil.getBean(IndexConfigService.class);
        IndexSetModel selectData = indexSetService.selectDetailData(this.indexLisModel);
        List<String> indexNoList = selectData.getIndexNos();
        List<IndexModel> indexModels = new ArrayList<>();
        for (String no : indexNoList) {
            IndexModel model = new IndexModel();
            model.setIndexNo(no);
            IndexModel data = indexConfigService.selectDetailData(model);
            if (null != data) {
                indexModels.add(data);
            }
        }
        return indexModels.stream().map(indexModel ->
                new IndexCalcUnit(indexModel.getIndexNo(),indexModel.getIndexName(),Map.of(),indexModel)).collect(Collectors.toList());
    }

    @Override
    public void afterCalc(Map<String, Object> context) {

    }

    @Override
    public void beforeCalc(Map<String, Object> context) {

    }

    @Override
    public void calc(CalcResult<String, Object> thisResult, String calculateInstanceNumber, Map<String, Object> context, Map<CalcUnit, CalcResult<String, Object>> sourceResults) {

        Map<String,Object> resMap = new HashMap<>();
        sourceResults.forEach(
            (calcUnit,result) -> {
                resMap.putAll(result);
            }
        );
        thisResult.put(this.getCalcCode(),resMap);
    }
}
