package com.sundata.internalevaluation.calc.calcUnit;


import cn.hutool.extra.spring.SpringUtil;
import com.sundata.internalevaluation.calc.model.CalcResult;
import com.sundata.internalevaluation.calc.model.CalcUnit;
import com.sundata.internalevaluation.calc.model.finals.CalcType;
import com.sundata.internalevaluation.configuration.model.IndexSetModel;
import com.sundata.internalevaluation.configuration.model.RiskControlStrategyModel;
import com.sundata.internalevaluation.configuration.model.RuleSet;
import com.sundata.internalevaluation.configuration.service.IndexSetService;
import com.sundata.internalevaluation.configuration.service.RiskControlStrategyService;
import com.sundata.internalevaluation.configuration.service.RuleService;
import com.sundata.internalevaluation.configuration.service.RuleSetService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

public class RiskControlStrategyCalcUnit extends CalcUnit {

    private static final Logger log = LoggerFactory.getLogger(RiskControlStrategyCalcUnit.class);

    RiskControlStrategyModel riskControlStrategyModel;
    /**
     * 创建数据单元的绝对对象，对象必须包含如下参数
     *
     * @param calcCode    计算对象编号
     * @param calcName    计算对象名称
     * @param calcType    计算类型
     * @param initContext 计算单元初始化参数
     */
    public RiskControlStrategyCalcUnit(String calcCode, String calcName, CalcType calcType, Map<String, Object> initContext, RiskControlStrategyModel riskControlStrategyModel) {
        super(calcCode, calcName, calcType, initContext);
        this.riskControlStrategyModel = riskControlStrategyModel;
    }

    @Override
    public boolean isCalcFinished(String calculateInstanceNumber) {
        return false;
    }

    @Override
    public void initResultContext(String calculateInstanceNumber) {

    }

    @Override
    public List<CalcUnit> getSourceCalcUnits() {
        List<IndexSetModel> indexSetModels = new ArrayList<>();
        List<RuleSet> sysReqRules = new ArrayList<>();
        List<CalcUnit> allListUnit = new ArrayList<>();
        RiskControlStrategyService riskControlStrategyService = SpringUtil.getBean(RiskControlStrategyService.class);
        IndexSetService indexSetService = SpringUtil.getBean(IndexSetService.class);
        RuleSetService ruleSetService = SpringUtil.getBean(RuleSetService.class);
        RuleService ruleService = SpringUtil.getBean(RuleService.class);
        RiskControlStrategyModel selectModel =  riskControlStrategyService.selectDetailData(this.riskControlStrategyModel);
        // 获取所有指标清单对象
        if (null != selectModel && null != selectModel.getIndexListNos() &&  selectModel.getIndexListNos().size()>0) {
            for(String indexListNo : selectModel.getIndexListNos()) {
                IndexSetModel model = new IndexSetModel();
                model.setIndexListNo(indexListNo);
                IndexSetModel afterSelectModel = indexSetService.selectDetailData(model);
                if (null != afterSelectModel) {
                    indexSetModels.add(afterSelectModel);
                }
            }
        }
        // 获取所有规则集对象
        if (null != selectModel && null != selectModel.getRulesNos() &&  selectModel.getRulesNos().size()>0) {
            for(String rulesNo : selectModel.getRulesNos()) {
                RuleSet afterSelectModel = ruleSetService.getById(rulesNo);
                if (null != afterSelectModel ) {
                    sysReqRules.add(afterSelectModel);
                }
            }
        }
        List<CalcUnit> indexListUnit = indexSetModels.stream().map(indexListModel -> new IndexListConfigCalcUnit(indexListModel.getIndexListNo(),indexListModel.getIndexListName(),CalcType.INDEXLIST,Map.of(),indexListModel)).collect(Collectors.toList());
        List<CalcUnit> rulesUnit = sysReqRules.stream().map(rulesModel -> new RulesCalcUnit(rulesModel.getRulesNo(),rulesModel.getRulesName(),Map.of(),rulesModel, ruleService)).collect(Collectors.toList());
        allListUnit.addAll(indexListUnit);
        allListUnit.addAll(rulesUnit);
        return allListUnit;
    }

    @Override
    public void afterCalc(Map<String, Object> context) {

    }

    @Override
    public void beforeCalc(Map<String, Object> context) {

    }

    @Override
    public void calc(CalcResult<String, Object> thisResult, String calculateInstanceNumber, Map<String, Object> context, Map<CalcUnit, CalcResult<String, Object>> sourceResults) {

//        Map<String,Object> resResultMap =new HashMap<>();
//        sourceResults.forEach(
//                (calcUnit,result) -> {
//                    resResultMap.putAll(result);
//        }
//        );
//        thisResult.put(this.getCalcCode(),resResultMap);
        Map<String,Object> rulesMap = new HashMap<>();
        Map<String,Object> indexListMap = new HashMap<>();
        rulesMap.put("approResult","1");
        List<Object> hitRuleList = new ArrayList<>();
        sourceResults.forEach(
                (calcUnit,result) -> {
                    if (calcUnit instanceof RulesCalcUnit) {
                        // 判断规则集是否触发拦截
                        Map<String,Object> rulesResMap = (Map<String,Object>) result.get(calcUnit.getCalcCode());
                        hitRuleList.addAll((List) rulesResMap.get("hitRule"));
                        Object approResult = rulesResMap.get("approResult");
                        if ("2".equals(approResult)) {
                            rulesMap.put("approResult","2");
                        }
                    } else if (calcUnit instanceof IndexListConfigCalcUnit) {
                        // 指标结果
                        Map<String,Object> indexResMap = (Map<String, Object>) result.get(calcUnit.getCalcCode());
                        indexListMap.put(calcUnit.getCalcCode(),indexResMap);
                    }
                }
        );
        log.debug("风控策略监测:命中规则集-[{}]-指标清单-[{}]",hitRuleList,indexListMap);
        // 结果封装
        rulesMap.put("hitRule",hitRuleList);
        thisResult.put("ruleResult",rulesMap);
        // 结果返回
        thisResult.put("indexResult",indexListMap);

    }
}
