package com.sundata.internalevaluation.calc.calcUnit.interfaces;

import com.sundata.common.util.JsonUtil;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * Created by IntelliJ IDEA.
 *
 * @author JoeLazy
 * @date 2025-04-28 08:54:32
 * @description: 接口处理案例
 */
@Service("addressInterfaceRunningService")
public class AddressInterfaceRunningService implements InterfaceRunning{
    /**
     * 加密逻辑
     *
     * @param interfaceStr 待处理字符串
     * @return 加密后逻辑
     */
    @Override
    public String encode(String interfaceStr) {
        return interfaceStr;
    }

    /**
     * 解密逻辑
     *
     * @param encodeString 待处理字符串
     * @return 解密后的逻辑
     */
    @Override
    public String decode(String encodeString) {
        return encodeString;
    }

    /**
     * 根据解密后的报文内容，对报文内容做出一定的处理
     *
     * @param interfaceNo             接口编号
     * @param interfaceStr            待处理字符串
     * @param calculateInstanceNumber 计算实例编号
     * @return 处理后的字符串报文
     */
    @Override
    public String afterDecode(String interfaceNo, String interfaceStr, String calculateInstanceNumber) {

        return JsonUtil.toJSONString(Map.of(interfaceNo, List.of(JsonUtil.jsonToMap(interfaceStr))));

//        // ************************ 举例
//        Map<String, Object> resMap = JSONUtil.toBean(interfaceStr, Map.class);
//        Object resCode = resMap.get("resCode");
//        // 判断是否请求成功返回正常的数据
//        if ("200".equals(resCode)){
//            // 解析数据的逻辑
//            // ......
//
//            // 返回解析后的数据
//            // return .... ;
//        }else{
//            throw new ResponseDataException(String.valueOf(resMap.get("resMsg")));
//        }
    }

    /**
     * 处理后的报文内容，若需要映射为对象，则对象内容应当具备初始化方法与getset 方法来提供映射逻辑，简而言之，类似于 将json 或 xml 转化为model对象
     *
     * @param interfaceStr 经过处理后的json 报文或 xml 报文
     * @param clazz        需要被转化的对象类型
     * @return 对象
     */
    @Override
    public <T> T coverObject(String interfaceStr, Class<T> clazz) {
        return null;
    }
}
