package com.sundata.internalevaluation.calc.util;

import cn.hutool.extra.spring.SpringUtil;
import com.sundata.internalevaluation.calc.calcUnit.RiskControlStrategyCalcUnit;
import com.sundata.internalevaluation.calc.calcUnit.RulesCalcUnit;
import com.sundata.internalevaluation.calc.model.CalcException;
import com.sundata.internalevaluation.calc.model.CalcUnit;
import com.sundata.internalevaluation.calc.model.finals.CalcType;
import com.sundata.internalevaluation.calc.running.CalcRunning;
import com.sundata.internalevaluation.calc.running.threads.CalcTaskResult;
import com.sundata.internalevaluation.configuration.model.RiskControlStrategyModel;
import com.sundata.internalevaluation.configuration.model.RuleSet;
import com.sundata.internalevaluation.configuration.service.RiskControlStrategyService;
import com.sundata.internalevaluation.configuration.service.RuleService;
import com.sundata.internalevaluation.configuration.service.RuleSetService;
import org.apache.commons.lang3.StringUtils;

import java.util.Map;

/**
 * 计算工具类
 */
public class CalcUtil {
    /**
     * 调用计算的过程
     * @param calculateInstanceNumber 计算流水号
     * @param calcUnit 计算节点
     * @param context 计算参数
     * @return 计算结果
     */
    public static CalcTaskResult calc(String calculateInstanceNumber, CalcUnit calcUnit, Map<String, Object> context) {
        return new CalcRunning().startCalc(calculateInstanceNumber, calcUnit, context);
    }

    /**
     * 计算规则
     * @param calculateInstanceNumber 计算流水号
     * @param rulesNo 规则集编号
     * @param context 参数  网贷传入的报文
     * @return 计算结果
     */
    public static CalcTaskResult calcRules(String calculateInstanceNumber, String rulesNo, Map<String, Object> context) {
        if (StringUtils.isBlank(rulesNo)) {
            throw new CalcException("数据集编号不能为空！！！");
        }
        RuleSetService rulesService = SpringUtil.getBean(RuleSetService.class);
        RuleSet rules = rulesService.getById(rulesNo);

        CalcUnit calcUnit = new RulesCalcUnit(rules.getRulesNo(),rules.getRulesName(),Map.of(),rules,SpringUtil.getBean(RuleService.class));
        CalcTaskResult calc = calc(calculateInstanceNumber, calcUnit, context);
        System.out.println(calc);
        return calc;
    }

    public static CalcTaskResult calcRiskCtrl(String calculateInstanceNumber, String riskCtrlNo, Map<String, Object> context) {
        if (StringUtils.isBlank(riskCtrlNo)) {
            throw new CalcException("数据集编号不能为空！！！");
        }
        RiskControlStrategyService riskControlStrategyService = SpringUtil.getBean(RiskControlStrategyService.class);


        RiskControlStrategyModel model = new RiskControlStrategyModel();
        model.setRiskControlStrategyNo(riskCtrlNo);
        RiskControlStrategyModel rules = riskControlStrategyService.selectDetailData(model);

        RiskControlStrategyCalcUnit calcUnit = new RiskControlStrategyCalcUnit(rules.getRiskControlStrategyNo(),rules.getRiskControlStrategyName(), CalcType.RISKCTRLSTRATEGY,Map.of(),rules);
        CalcTaskResult calc = calc(calculateInstanceNumber, calcUnit, context);
        System.out.println(calc);
        return calc;
    }


}
