package com.sundata.internalevaluation.configuration.controller;


import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.sundata.common.base.BaseAction;
import com.sundata.common.exception.BusinessException;
import com.sundata.internalevaluation.calc.running.threads.CalcTaskResult;
import com.sundata.internalevaluation.calc.util.CalcUtil;
import com.sundata.internalevaluation.configuration.model.RiskControlStrategyModel;
import com.sundata.internalevaluation.configuration.service.RiskControlStrategyService;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/internalevaluation/configuration/RiskControlStrategyAction")
public class RiskControlStrategyAction extends BaseAction {
    
    @Autowired
    RiskControlStrategyService riskControlStrategyService;

    /**
     * @author: 张艳杰
     * @methodsName: getDataList
     * @description: 风控策略配置初始化
     * @param: HttpServletRequest request, HttpServletResponse response,@RequestBody RiskControlStrategyModel model
     * @return: List<RiskControlStrategyModel>
     * @throws:
     */
    @RequestMapping({"getDataList"})
    public List<RiskControlStrategyModel> getDataList(HttpServletRequest request, HttpServletResponse response,
                                            @RequestBody RiskControlStrategyModel model) {
        return riskControlStrategyService.getDataList(model);
    }

    /**
     * @author: 张艳杰
     * @methodsName: createData
     * @description: 风控策略配置新增
     * @param: HttpServletRequest request, HttpServletResponse response,@RequestBody RiskControlStrategyModel model
     * @return:
     * @throws:
     */
    @RequestMapping({"createData"})
    public void createData(HttpServletRequest request, HttpServletResponse response,
                           @RequestBody RiskControlStrategyModel model) {
        riskControlStrategyService.createData(model);
    }

    /**
     * @author: 张艳杰
     * @methodsName: deleteRows
     * @description: 风控策略配置删除
     * @param: HttpServletRequest request, HttpServletResponse response,@RequestBody RiskControlStrategyModel model
     * @return:
     * @throws:
     */
    @RequestMapping({"deleteRows"})
    public void deleteRows(HttpServletRequest request, HttpServletResponse response,
                           @RequestBody RiskControlStrategyModel model) {
        riskControlStrategyService.deleteRows(model);
    }

    /**
     * @author: 张艳杰
     * @methodsName: updateExistData
     * @description: 风控策略配置修改
     * @param: HttpServletRequest request, HttpServletResponse response,@RequestBody RiskControlStrategyModel model
     * @return:
     * @throws:
     */
    @RequestMapping({"updateExistData"})
    public void updateExistData(HttpServletRequest request, HttpServletResponse response,
                                @RequestBody RiskControlStrategyModel model) {
        riskControlStrategyService.updateExistData(model);
    }


    /**
     * @author: 张艳杰
     * @methodsName: selectDetailData
     * @description: 风控策略配置查询
     * @param: HttpServletRequest request, HttpServletResponse response,@RequestBody RiskControlStrategyModel model
     * @return:
     * @throws:
     */
    @RequestMapping({"selectDetailData"})
    public RiskControlStrategyModel selectDetailData(HttpServletRequest request, HttpServletResponse response,
                                           @RequestBody RiskControlStrategyModel model) {
        return riskControlStrategyService.selectDetailData(model);
    }


    @RequestMapping("/testRiskCtrl")
    public CalcTaskResult testRiskCtrl(@RequestBody TestRiskCtrlModel riskCtrlModel){
        String paramJsonStr = riskCtrlModel.getParamJsonStr();
        Map<String, Object> paramMap;
        if (StrUtil.isNotBlank(paramJsonStr)){
            paramMap = JSONUtil.toBean(paramJsonStr, Map.class);
        }else{
            throw new BusinessException("参数未传！！");
        }
        CalcTaskResult calcTaskResult = null;
        try {
            calcTaskResult = CalcUtil.calcRiskCtrl("", riskCtrlModel.getRiskCtrlNo(), paramMap);
        } catch (Exception e) {
            throw new BusinessException(e.getMessage(),e);
        }
        return calcTaskResult;
    }

    public static class TestRiskCtrlModel{
        private String riskCtrlNo;

        private String paramJsonStr;

        private Map<String, Object> paramMap;

        public String getRiskCtrlNo() {
            return riskCtrlNo;
        }

        public void setRiskCtrlNo(String riskCtrlNo) {
            this.riskCtrlNo = riskCtrlNo;
        }

        public String getParamJsonStr() {
            return paramJsonStr;
        }

        public void setParamJsonStr(String paramJsonStr) {
            this.paramJsonStr = paramJsonStr;
        }

        public Map<String, Object> getParamMap() {
            return paramMap;
        }

        public void setParamMap(Map<String, Object> paramMap) {
            this.paramMap = paramMap;
        }

        @Override
        public String toString() {
            return "TestRiskCtrlModel{" +
                    "riskCtrlNo='" + riskCtrlNo + '\'' +
                    ", paramJsonStr='" + paramJsonStr + '\'' +
                    ", paramMap=" + paramMap +
                    '}';
        }
    }


}
