package com.sundata.internalevaluation.configuration.controller;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.sundata.common.base.BaseAction;
import com.sundata.internalevaluation.calc.util.CalcUtil;
import com.sundata.internalevaluation.configuration.model.RuleSet;
import com.sundata.internalevaluation.configuration.service.RuleSetService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/nprules")
public class RuleSetController extends BaseAction {

    @Autowired
    private RuleSetService ruleSetService;

    /**
     * 查询所有记录
     *
     * @return 返回集合，没有返回空List
     */
    @RequestMapping("list")
    public List<RuleSet> listAll(@RequestBody RuleSet ruleSet) {
        return ruleSetService.listAll(ruleSet);
    }


    /**
     * 根据主键查询
     *
     * @param id 主键
     * @return 返回记录，没有返回null
     */
    @RequestMapping("getById")
    public RuleSet getById(@RequestBody RuleSet ruleSet) {
        return ruleSetService.getById(ruleSet.getRulesNo());
    }    
     
    /**
     * 新增，忽略null字段
     *
     * @param ruleSet 新增的记录
     * @return 返回影响行数
     */
    @RequestMapping("insert")
    public int insert(@RequestBody RuleSet ruleSet) throws JsonProcessingException {
        return ruleSetService.insertIgnoreNull(ruleSet);
    }    
      
    /**
     * 修改，忽略null字段
     *
     * @param ruleSet 修改的记录
     * @return 返回影响行数
     */
    @RequestMapping("update")
    public int update(@RequestBody RuleSet ruleSet) {
        return ruleSetService.updateIgnoreNull(ruleSet);
    }
    
    /**
     * 删除记录
     *
     * @param ruleSet 待删除的记录
     * @return 返回影响行数
     */
    @RequestMapping("delete")
    public int delete(@RequestBody RuleSet ruleSet) {
        return ruleSetService.delete(ruleSet);
    }


    @RequestMapping("testRules")
    public Object testRules(@RequestBody RuleSet ruleSet) {
        return CalcUtil.calcRules("rules"+System.currentTimeMillis(), ruleSet.getRulesNo(),
                Map.of("name","zhangsan","custNo","1"));
    }
    
}