package com.sundata.internalevaluation.configuration.service;

import com.sundata.internalevaluation.configuration.mapper.DataSetConfigMapper;
import com.sundata.internalevaluation.configuration.model.DataSetModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @author JoeLazy
 */
@Service
public class DataSetService {
    static final Logger log = LoggerFactory.getLogger(DataSetService.class);

    @Autowired
    DataSetConfigMapper dataSetConfigMapper;

    /**
     * 查询数据集列表
     *
     * @param model 查询条件
     * @return 列表
     */
    public List<DataSetModel> getDataList(DataSetModel model) {
        return dataSetConfigMapper.getDataList(model);
    }

    /**
     * 数据集配置新增
     * @param model 新增数据
     */
    public void createData(DataSetModel model) {
        if (null != model && null != model.getDataSourcesNos() && model.getDataSourcesNos().size()>0) {
            for (String sourceNo : model.getDataSourcesNos()) {
                model.setDataSourcesNo(sourceNo);
                dataSetConfigMapper.createAssociation(model);
            }
        }
        dataSetConfigMapper.createData(model);
    }

    /**
     * 数据集配置删除
     * @param model 条件
     */
    public void deleteRows(DataSetModel model) {
        dataSetConfigMapper.deleteAssociation(model.getDataSetNo());
        dataSetConfigMapper.deleteRows(model);
    }

    /**
     * 数据集配置修改
     * @param model 实体
     */
    public void updateExistData(DataSetModel model) {
        dataSetConfigMapper.deleteAssociation(model.getDataSetNo());
        if (null != model.getDataSourcesNos() && model.getDataSourcesNos().size()>0) {
            for (String sourceNo : model.getDataSourcesNos()) {
                model.setDataSourcesNo(sourceNo);
                dataSetConfigMapper.createAssociation(model);
            }
        }
        dataSetConfigMapper.updateExistData(model);
    }

    /**
     * 数据来源配置查询
     * @param model 条件
     * @return 结果
     */
    public DataSetModel selectDetailData(DataSetModel model) {
        DataSetModel configModel = dataSetConfigMapper.selectDetailData(model);
        List<String> dataSourcesNos = new ArrayList<>();
        List<DataSetModel> selectModel = dataSetConfigMapper.selectAssociation(configModel);
        if (null != selectModel && selectModel.size()>0) {
            for (DataSetModel m : selectModel) {
                dataSourcesNos.add(m.getDataSourcesNo());
            }
        }
        configModel.setDataSourcesNos(dataSourcesNos);
        return configModel;
    }
}
