package com.sundata.internalevaluation.configuration.service;

import com.sundata.internalevaluation.configuration.mapper.DataSourcesMapper;
import com.sundata.internalevaluation.configuration.model.DataItemConfigModel;
import com.sundata.internalevaluation.configuration.model.DataSourceModel;
import com.sundata.internalevaluation.configuration.model.InputParamModel;
import com.sundata.internalevaluation.configuration.model.QueryLogicModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class DataSourcesService {

    @Autowired
    DataSourcesMapper dataSourcesMapper;

    /**
     * 数据来源配置初始化
     *
     * @param model 查询条件
     * @return 列表
     */
    public List<DataSourceModel> getDataList(DataSourceModel model) {
        return dataSourcesMapper.getDataList(model);
    }


    /**
     * 数据来源配置新增
     *
     * @param model 新增数据
     */
    public void createData(DataSourceModel model) {
        // 先插入数据来源信息
        dataSourcesMapper.createData(model);
        // 插入输入参数信息
        insertInputParamAndDataItemAndQueryLogic(model);
    }

    /**
     * 插入查询逻辑信息
     * @param queryLogicList 输入参数列表
     * @return 影响行数
     */
    public int createQueryLogicData(List<QueryLogicModel> queryLogicList){
        if (queryLogicList == null || queryLogicList.isEmpty()){
            return 0;
        }
        int rows = 0;
        for (QueryLogicModel queryLogic : queryLogicList) {
            rows += dataSourcesMapper.createQueryLogicData(queryLogic);
        }
        return rows;
    }

    /**
     * 插入数据项信息
     * @param dataItemList 输入参数列表
     * @return 影响行数
     */
    public int createDataItemConfigData(List<DataItemConfigModel> dataItemList){
        if (dataItemList == null || dataItemList.isEmpty()){
            return 0;
        }
        int rows = 0;
        for (DataItemConfigModel dataItem : dataItemList) {
            rows += dataSourcesMapper.createDataItemConfigData(dataItem);
        }
        return rows;
    }


    /**
     * 插入输入参数信息
     * @param inputParamList 输入参数列表
     * @return 影响行数
     */
    public int createInputParamData(List<InputParamModel> inputParamList){
        if (inputParamList == null || inputParamList.isEmpty()){
            return 0;
        }
        int rows = 0;
        for (InputParamModel inputParam : inputParamList) {
            rows += dataSourcesMapper.createInputParamData(inputParam);
        }
        return rows;
    }


    /**
     * 数据来源配置删除
     *
     * @param model 删除条件
     */
    public void deleteRows(DataSourceModel model) {
        dataSourcesMapper.deleteRows(model);
    }


    /**
     * 数据来源配置修改
     *
     * @param model 数据实体
     */
    public void updateExistData(DataSourceModel model) {
        dataSourcesMapper.updateExistData(model);
        dataSourcesMapper.deleteInputParamRows(model);
        dataSourcesMapper.deleteDataSourceRows(model);
        dataSourcesMapper.deleteQueryLogicRows(model);
        // 插入输入参数信息
        insertInputParamAndDataItemAndQueryLogic(model);
    }


    /**
     * 插入输入参数、数据项、查询逻辑等信息
     * @param model 数据来源实体
     */
    public void insertInputParamAndDataItemAndQueryLogic(DataSourceModel model) {
        // TODO 输入参数不需要关联数据源？？
//        this.createInputParamData(model.getInputParam());

        if (null != model.getInputParam() && !model.getInputParam().isEmpty()) {
            for (InputParamModel m : model.getInputParam()) {
                m.setDataSourcesNo(model.getDataSourcesNo());
            }
            this.createInputParamData(model.getInputParam());
        }

        if (null != model.getDataItemConfig() && !model.getDataItemConfig().isEmpty()) {
            for (DataItemConfigModel m : model.getDataItemConfig()) {
                m.setDataSourcesNo(model.getDataSourcesNo());
            }
            this.createDataItemConfigData(model.getDataItemConfig());
        }

        if (null != model.getQueryLogic() && !model.getQueryLogic().isEmpty()) {
            for (QueryLogicModel m : model.getQueryLogic()) {
                m.setDataSourcesNo(model.getDataSourcesNo());
            }
            this.createQueryLogicData(model.getQueryLogic());
        }
    }


    /**
     * 数据来源配置修改
     *
     * @param dataSourcesNo 数据源编号
     * @return 查询结果
     */
    public DataSourceModel selectDetailData(String dataSourcesNo) {
        DataSourceModel model = dataSourcesMapper.selectDetailData(dataSourcesNo);
        List<InputParamModel> inputList = dataSourcesMapper.getInputParamList(dataSourcesNo);
        List<DataItemConfigModel> dataItemList = dataSourcesMapper.getDataItemConfigList(dataSourcesNo);
        List<QueryLogicModel> queryLogicList = dataSourcesMapper.getQueryLogicList(dataSourcesNo);
        model.setInputParam(inputList);
        model.setQueryLogic(queryLogicList);
        model.setDataItemConfig(dataItemList);
        return model;
    }
}
