package com.sundata.internalevaluation.configuration.service;

import com.sundata.internalevaluation.configuration.mapper.IndexConfigMapper;
import com.sundata.internalevaluation.configuration.model.IndexModel;
import com.sundata.internalevaluation.configuration.model.IndexSourceModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.UUID;

/**
 * @author JoeLazy
 */
@Service
public class IndexConfigService {

    @Autowired
    IndexConfigMapper indexConfigMapper;


    /**
     * 指标配置初始化
     *
     * @param model 查询条件
     * @return 结果列表
     */
    public List<IndexModel> getDataList(IndexModel model) {
        return indexConfigMapper.getDataList(model);
    }


    /**
     * 指标配置新增
     *
     * @param model 新增内容
     */
    public void createData(IndexModel model) {
        indexConfigMapper.createData(model);
        if (null != model.getIndexDataSource() && !model.getIndexDataSource().isEmpty()) {
            indexConfigMapper.deleteIndexSource(model.getIndexNo());
            for (IndexSourceModel m : model.getIndexDataSource()) {
                m.setIndexNo(model.getIndexNo());
                m.setIndexSourceNo(UUID.randomUUID().toString());
            }
            this.createIndexSource(model.getIndexDataSource());
        }
    }

    /**
     * 插入指标
     * @param indexSource IndexSourceModel列表
     * @return 影响行数
     */
    public int createIndexSource(List<IndexSourceModel> indexSource) {
        if (indexSource == null || indexSource.isEmpty()) {
            return 0;
        }
        int rows = 0;
        for (IndexSourceModel indexSourceModel : indexSource) {
            rows += indexConfigMapper.createIndexSource(indexSourceModel);
        }
        return rows;
    }


    /**
     * 指标配置删除
     *
     * @param model 删除条件
     */
    public void deleteRows(IndexModel model) {
        indexConfigMapper.deleteRows(model);
        indexConfigMapper.deleteIndexSource(model.getIndexNo());
    }


    /**
     * 指标配置修改
     *
     * @param model 修改内容
     */
    public void updateExistData(IndexModel model) {
        indexConfigMapper.updateExistData(model);
        if (null != model.getIndexDataSource() && !model.getIndexDataSource().isEmpty()) {
            indexConfigMapper.deleteIndexSource(model.getIndexNo());
            for (IndexSourceModel m : model.getIndexDataSource()) {
                m.setIndexNo(model.getIndexNo());
                m.setIndexSourceNo(UUID.randomUUID().toString());
            }
            this.createIndexSource(model.getIndexDataSource());
        }else{
            indexConfigMapper.deleteIndexSource(model.getIndexNo());
        }
    }


    /**
     * 指标配置查询
     *
     * @param model 查培条件
     * @return 查询结果
     */
    public IndexModel selectDetailData(IndexModel model) {
        IndexModel selectDataSource = indexConfigMapper.selectDetailData(model);

        List<IndexSourceModel> sourceList = indexConfigMapper.getIndexSourceList(selectDataSource);
        selectDataSource.setIndexDataSource(sourceList);
        return selectDataSource;
    }

    public List<IndexSourceModel> getIndexSourceList(IndexModel indexModel) {
        return indexConfigMapper.getIndexSourceList(indexModel);
    }

    public List<IndexModel> selectIndexListByRuleNo(String ruleNo) {
        return indexConfigMapper.selectIndexListByRuleNo(ruleNo);
    }

    /**
     * 指标配置主键冲突查询
     * @param model 查询条件
     * @return 查询结果
     */
    public IndexModel isNotExist(IndexModel model) {
        return indexConfigMapper.selectDetailData(model);
    }
}
