package com.sundata.internalevaluation.configuration.service;

import com.sundata.common.exception.BusinessException;
import com.sundata.internalevaluation.configuration.mapper.InterfaceParamMapper;
import com.sundata.internalevaluation.configuration.model.InterfaceParam;
import org.apache.ibatis.session.SqlSessionFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

@Service
public class InterfaceParamService {

    @Autowired
    private InterfaceParamMapper interfaceParamMapper;

    @Autowired
    private SqlSessionFactory sqlSessionFactory;

    /**
     * 根据接口编号查询参数列表
     *
     * @param interfaceNo 主键
     * @return 返回记录，没有返回null
     */
    public List<InterfaceParam> getByInterfaceNo(String interfaceNo) {
        return interfaceParamMapper.getByInterfaceNo(interfaceNo);
    }

    /**
     * 根据接口id删除参数信息
     *
     * @param interfaceNo 主键
     * @return 返回记录，没有返回null
     */
    public int deleteByInterfaceNo(String interfaceNo) {
        return interfaceParamMapper.deleteByInterfaceNo(interfaceNo);
    }

    /**
     * 批量插入
     *
     * @param paramList 新增的记录
     * @return 返回影响行数
     */
    public int insertBatch(List<InterfaceParam> paramList, String interfaceNo) {
        if (paramList == null || paramList.isEmpty()) {
            return 1;
        }
        AtomicInteger rows= new AtomicInteger();
        paramList.forEach(param -> {
            param.setParamObjName(param.getParamObjName().trim());
            param.setInterfaceNo(interfaceNo);
            try {
                rows.addAndGet(insert(param));
            } catch (DuplicateKeyException e) {
                throw new BusinessException("相同变量名只能存在一个 ["+ param.getParamObjName() +"]");
            }
        });
        return rows.get();
    }


    /**
     * 新增，插入所有字段
     *
     * @param interfaceParam 新增的记录
     * @return 返回影响行数
     */
    public int insert(InterfaceParam interfaceParam) {
        return interfaceParamMapper.insertIgnoreNull(interfaceParam);
    }

    /**
     * 新增，忽略null字段
     *
     * @param interfaceParam 新增的记录
     * @return 返回影响行数
     */
    public int insertIgnoreNull(InterfaceParam interfaceParam) {
        return interfaceParamMapper.insertIgnoreNull(interfaceParam);
    }

    /**
     * 修改，修改所有字段
     *
     * @param interfaceParam 修改的记录
     * @return 返回影响行数
     */
    public int update(InterfaceParam interfaceParam) {
        return interfaceParamMapper.update(interfaceParam);
    }

}