package com.sundata.internalevaluation.configuration.service;

import cn.hutool.core.util.StrUtil;
import com.sundata.common.base.ActiveUserService;
import com.sundata.common.exception.BusinessException;
import com.sundata.common.login.ActiveUser;
import com.sundata.common.util.DateUtil;
import com.sundata.internalevaluation.calc.util.IdUtils;
import com.sundata.internalevaluation.configuration.mapper.InterfaceMapper;
import com.sundata.internalevaluation.configuration.model.Interface;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author JoeLazy
 */
@Service
public class InterfaceService {

    @Autowired
    private InterfaceMapper interfaceMapper;


    @Autowired
    private InterfaceParamService paramService;

    /**
     * 查询所有记录
     *
     * @return 返回集合，没有返回空List
     */
    public List<Interface> listAll(Interface interfaceModel) {
    	return interfaceMapper.listAll(interfaceModel);
    }


    /**
     * 根据主键查询
     *
     * @param id 主键
     * @return 返回记录，没有返回null
     */
    public Interface getById(String id) {
        return interfaceMapper.getById(id);
    }
	
    /**
     * 新增，忽略null字段
     *
     * @param interfaceModel 新增的记录
     * @return 返回影响行数
     */

    @Transactional
    public int insertIgnoreNull(Interface interfaceModel) {
        String interfaceNo = interfaceModel.getInterfaceNo();
        if (StrUtil.isBlankIfStr(interfaceNo)){
            throw new BusinessException("接口编号不能为空");
        }
        interfaceNo = interfaceNo.trim();
        interfaceModel.setInterfaceNo(interfaceNo);
        Long snowflakeId = IdUtils.getSnowflakeId();
        interfaceModel.setId(snowflakeId.toString());
        ActiveUser activeUser = ActiveUserService.readActiveUser();
        String account = activeUser.getAccount();
        interfaceModel.setCreateBy(account);
        interfaceModel.setUpdateBy(account);
        String nowTime = DateUtil.getCurTime();
        interfaceModel.setCreateTime(nowTime);
        interfaceModel.setUpdateTime(nowTime);
        paramService.deleteByInterfaceNo(interfaceNo);
        paramService.insertBatch(interfaceModel.getParamList(), interfaceNo);
        try {
            return interfaceMapper.insertIgnoreNull(interfaceModel);
        } catch (DuplicateKeyException e) {
            throw new BusinessException("接口编号[ "+interfaceNo+" ]已存在，请重新输入",e);
        }
    }
	
    /**
     * 修改，修改所有字段
     *
     * @param interfaceModel 修改的记录
     * @return 返回影响行数
     */
    @Transactional
    public int update(Interface interfaceModel) {
        String interfaceNo = interfaceModel.getInterfaceNo();
        if (StrUtil.isBlankIfStr(interfaceNo)){
            throw new BusinessException("接口编号不能为空");
        }
        interfaceNo = interfaceNo.trim();
        ActiveUser activeUser = ActiveUserService.readActiveUser();
        String account = activeUser.getAccount();
        interfaceModel.setUpdateBy(account);
        String nowTime = DateUtil.getCurTime();
        interfaceModel.setUpdateTime(nowTime);
        paramService.deleteByInterfaceNo(interfaceNo);
        paramService.insertBatch(interfaceModel.getParamList(), interfaceNo);
    	return interfaceMapper.update(interfaceModel);
    }

    /**
     * 删除记录
     *
     * @param interfaceModel 待删除的记录
     * @return 返回影响行数
     */
    @Transactional
    public int delete(Interface interfaceModel) {
        paramService.deleteByInterfaceNo(interfaceModel.getInterfaceNo());
    	return interfaceMapper.delete(interfaceModel);
    }
	
}