package com.sundata.internalevaluation.configuration.service;

import com.sundata.internalevaluation.configuration.mapper.RiskControlStrategyMapper;
import com.sundata.internalevaluation.configuration.model.RiskControlStrategyModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @author JoeLazy
 */
@Service
public class RiskControlStrategyService {

    @Autowired
    RiskControlStrategyMapper riskControlStrategyMapper;

    /**
     * 风控策略配置初始化
     *
     * @param model 查询条件
     * @return 查询列表
     */
    public List<RiskControlStrategyModel> getDataList(RiskControlStrategyModel model) {
        return riskControlStrategyMapper.getDataList(model);
    }


    /**
     * 风控策略配置新增
     *
     * @param model 新增内容
     */
    public void createData(RiskControlStrategyModel model) {
        // 插入对应的风控规则和指标清单关联
        if (null != model && null != model.getIndexListNos() && !model.getIndexListNos().isEmpty()) {
            for (String index : model.getIndexListNos()) {
                model.setIndexListNo(index);
                riskControlStrategyMapper.createIndexListAssociation(model);
            }
        }
        // 插入对应的风控规则和指标清单关联
        if (null != model && null != model.getRulesNos() && !model.getRulesNos().isEmpty()) {
            for (String rules : model.getRulesNos()) {
                model.setRulesNo(rules);
                riskControlStrategyMapper.createRulesAssociation(model);
            }
        }
        // 规则表插入数据
        riskControlStrategyMapper.createData(model);
    }


    /**
     * 风控策略配置删除
     *
     * @param model 删除条件
     */
    public void deleteRows(RiskControlStrategyModel model) {
        riskControlStrategyMapper.deleteIndexListAssociation(model);
        riskControlStrategyMapper.deleteRulesAssociation(model);
        riskControlStrategyMapper.deleteRows(model.getRiskControlStrategyNo());
    }


    /**
     * 风控策略配置修改
     *
     * @param model 修改内容
     */
    public void updateExistData(RiskControlStrategyModel model) {
        riskControlStrategyMapper.updateExistData(model);
        // 插入对应的风控规则和指标清单关联
        if (null != model && null != model.getIndexListNos() && !model.getIndexListNos().isEmpty()) {
            riskControlStrategyMapper.deleteIndexListAssociation(model);
            for (String index : model.getIndexListNos()) {
                model.setIndexListNo(index);
                riskControlStrategyMapper.createIndexListAssociation(model);
            }

        }else{
            riskControlStrategyMapper.deleteIndexListAssociation(model);
        }
        // 插入对应的风控规则和指标清单关联
        if (null != model && null != model.getRulesNos() && !model.getRulesNos().isEmpty()) {
            riskControlStrategyMapper.deleteRulesAssociation(model);
            for (String rules : model.getRulesNos()) {
                model.setRulesNo(rules);
                riskControlStrategyMapper.createRulesAssociation(model);
            }
        }else{
            riskControlStrategyMapper.deleteRulesAssociation(model);
        }
    }


    /**
     * 风控策略配置查询
     *
     * @param model 查询条件
     * @return 查询结果
     */
    public RiskControlStrategyModel selectDetailData(RiskControlStrategyModel model) {
        List<String> rulesList = new ArrayList<>();
        List<String> indexList = new ArrayList<>();
        RiskControlStrategyModel selectModel = riskControlStrategyMapper.selectDetailData(model);
        List<RiskControlStrategyModel> associationList = riskControlStrategyMapper.selectIndexListAssociation(selectModel.getRiskControlStrategyNo());
        List<RiskControlStrategyModel> rulesAssociationList = riskControlStrategyMapper.selectRulesAssociationData(selectModel.getRiskControlStrategyNo());
        associationList.addAll(rulesAssociationList);
        if (null != associationList && !associationList.isEmpty()) {
            for (RiskControlStrategyModel m : associationList) {
                if (null != m.getRulesNo() && !m.getRulesNo().isEmpty()) {
                    rulesList.add(m.getRulesNo());
                }
                if (null != m.getIndexListNo() && !m.getIndexListNo().isEmpty()) {
                    indexList.add(m.getIndexListNo());
                }
            }
            selectModel.setIndexListNos(indexList);
            selectModel.setRulesNos(rulesList);
        }
        return selectModel;
    }
}
