package com.sundata.internalevaluation.script.model;

import cn.hutool.core.util.StrUtil;
import org.jetbrains.annotations.Nullable;

import java.util.Map;
import java.util.Objects;
import java.util.StringJoiner;

public class ScriptRunningModel {
    private String runningId;
    private ScriptModel scriptModel;
    private Map<String,Object> variables;

    public ScriptRunningModel(@Nullable String runningId,@Nullable ScriptModel scriptModel,@Nullable Map<String, Object> variables) {
        if (StrUtil.isEmpty(runningId)) {
            throw new IllegalArgumentException("runningId is null");
        }
        if (scriptModel == null) {
            throw new IllegalArgumentException("scriptModel is null");
        }
        if (variables == null) {
            throw new IllegalArgumentException("variables is null");
        }
        this.runningId = runningId;
        this.scriptModel = scriptModel;
        this.variables = variables;
    }

    @Override
    public String toString() {
        return new StringJoiner(", ", ScriptRunningModel.class.getSimpleName() + "[", "]")
                .add("runningId='" + runningId + "'")
                .add("scriptModel=" + scriptModel)
                .add("variables=" + variables)
                .toString();
    }

    @Override
    public boolean equals(Object o) {
        if (o == null || getClass() != o.getClass()) return false;

        ScriptRunningModel that = (ScriptRunningModel) o;
        return Objects.equals(getRunningId(), that.getRunningId()) && Objects.equals(getScript(), that.getScript()) && Objects.equals(getVariables(), that.getVariables());
    }

    @Override
    public int hashCode() {
        int result = Objects.hashCode(getRunningId());
        result = 31 * result + Objects.hashCode(getScript());
        result = 31 * result + Objects.hashCode(getVariables());
        return result;
    }

    public String getRunningId() {
        return runningId;
    }

    public void setRunningId(String runningId) {
        this.runningId = runningId;
    }

    public ScriptModel getScript() {
        return scriptModel;
    }

    public void setScript(ScriptModel scriptModel) {
        this.scriptModel = scriptModel;
    }

    public Map<String, Object> getVariables() {
        return variables;
    }

    public void setVariables(Map<String, Object> variables) {
        this.variables = variables;
    }

}
